'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.buildSnapshotResolver = exports.isSnapshotPath = exports.DOT_EXTENSION = exports.EXTENSION = undefined;

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

// Copyright (c) 2014-present, Facebook, Inc. All rights reserved.

const EXTENSION = (exports.EXTENSION = 'snap');
const DOT_EXTENSION = (exports.DOT_EXTENSION = '.' + EXTENSION);

const isSnapshotPath = (exports.isSnapshotPath = path =>
  path.endsWith(DOT_EXTENSION));

const cache = new Map();
const buildSnapshotResolver = (exports.buildSnapshotResolver = config => {
  const key = config.rootDir;
  if (!cache.has(key)) {
    cache.set(key, createSnapshotResolver(config.snapshotResolver));
  }
  return cache.get(key);
});

function createSnapshotResolver(snapshotResolverPath) {
  return typeof snapshotResolverPath === 'string'
    ? createCustomSnapshotResolver(snapshotResolverPath)
    : {
        resolveSnapshotPath: testPath =>
          _path2.default.join(
            _path2.default.join(
              _path2.default.dirname(testPath),
              '__snapshots__'
            ),
            _path2.default.basename(testPath) + DOT_EXTENSION
          ),

        resolveTestPath: snapshotPath =>
          _path2.default.resolve(
            _path2.default.dirname(snapshotPath),
            '..',
            _path2.default.basename(snapshotPath, DOT_EXTENSION)
          )
      };
}

function createCustomSnapshotResolver(snapshotResolverPath) {
  const custom = require(snapshotResolverPath);

  if (typeof custom.resolveSnapshotPath !== 'function') {
    throw new TypeError(mustImplement('resolveSnapshotPath'));
  }
  if (typeof custom.resolveTestPath !== 'function') {
    throw new TypeError(mustImplement('resolveTestPath'));
  }

  const customResolver = {
    resolveSnapshotPath: testPath =>
      custom.resolveSnapshotPath(testPath, DOT_EXTENSION),
    resolveTestPath: snapshotPath =>
      custom.resolveTestPath(snapshotPath, DOT_EXTENSION)
  };

  verifyConsistentTransformations(customResolver);

  return customResolver;
}

function mustImplement(functionName) {
  return (
    _chalk2.default.bold(
      `Custom snapshot resolver must implement a \`${functionName}\` function.`
    ) +
    '\nDocumentation: https://facebook.github.io/jest/docs/en/configuration.html#snapshotResolver'
  );
}

function verifyConsistentTransformations(custom) {
  const fakeTestPath = _path2.default.posix.join(
    'some-path',
    '__tests__',
    'snapshot_resolver.test.js'
  );
  const transformedPath = custom.resolveTestPath(
    custom.resolveSnapshotPath(fakeTestPath)
  );
  if (transformedPath !== fakeTestPath) {
    throw new Error(
      _chalk2.default.bold(
        `Custom snapshot resolver functions must transform paths consistently, i.e. expects resolveTestPath(resolveSnapshotPath('${fakeTestPath}')) === ${transformedPath}`
      )
    );
  }
}
